package maya;

import java.util.*;

public class CU{
	public static final int[] LEN = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31,
			30, 31};
	public static final int[] LEAP_LEN = {31, 29, 31, 30, 31, 30, 31, 31, 30,
			31, 30, 31};
	public static final int TZOLKIN_DAYS = 13;
	public static final int TZOLKIN_MONTHS = 20;
	public static final int TZOLKIN_YEAR_LENGTH = 260;
	public static final String[] TZOLKIN_MMS = {"Imix", "Ik", "Akbal", "Kan",
			"Chicchan", "Cimi", "Manik", "Lamat", "Muluc", "Oc", "Chuen", "Eb",
			"Ben", "Ix", "Men", "Cib", "Caban", "Etznab", "Cauac", "Ahau"};
	public static final int MAX_UAYEB_DAY_NO = 4;
	public static final int HAAB_MAX_DAY_NO = 19;
	public static final int HAAB_DAYS = 20;
	public static final int HAAB_YEAR_LENGTH = 365;
	public static final String[] HAAB_MMS = {"Pop", "Uo", "Zip", "Zotz", "Zec",
			"Xul", "Yaxkin", "Mol", "Chen", "Yax", "Zac", "Ceh", "Mac",
			"Kankin", "Muan", "Pax", "Kayab", "Cumku", "Uayeb"};
	public static final int[] LC_VALUES = {144000, 7200, 360, 20, 1};
	public static final int[] LC_MAX = {13, 19, 19, 17, 19};
	public static final int MAX_LC_VALUE = 1898000;
	public static final Tzolkin TZSTART = new Tzolkin(4, "Ahau");
	public static final Haab HASTART = new Haab(8, "Cumku");
	public static final int CR_LENGTH = 18980;
	public static final double JD0 = 0.0;
	public static final YMD JD0_YMD = new YMD(-4712, 1, 1.5);
	public static final double FIRST_MAYA_JD = 584283;
	public static final YMD FIRST_MAYA_YMD = new YMD(-3113, 9, 6.5);

	private CU(){}

	//------------------------------ YMD
	public static boolean isLeapYear(int rok) {
		if(rok == 1582){
			return false;
		}
		if(rok < 1582){
                    return (rok % 4) == 0;
		}
		if(!(String.valueOf(rok)).endsWith("00")){
return (rok % 4) == 0;
		}
		else{
return (rok % 400) == 0;
		}
	}

	public static double dateToJulianDay(int rok, int miesiac, double dzien) {
		int y = rok;
		int m = miesiac;
		int a;
		int b = 0;
		if(m < 3){
			y--;
			m += 12;
		}
		if(isGCDate(rok, miesiac, dzien)){
			a = y / 100;
			b = 2 - a + (a / 4);
		}
		return (int)(365.25 * (y + 4716)) + (int)(30.6001 * (m + 1)) + dzien
				+ b - 1524.5;
	}

	/**
	 * Sprawdza czy podana data jest datą w Gregorian Calendar
	 * @return - zwraca true, jeśli podana data jest datą gregoriańską, a
	 * false, jeśli jest datą juliańską.
	 */
	public static boolean isGCDate(int rok, int miesiac, double dzien) {
		if(rok > 1582){
			return true;
		}
		if(rok < 1582){
			return false;
		}
		if(miesiac > 10){
			return true;
		}
		if(miesiac < 10){
			return false;
		}
		if(dzien >= 15.5){
			return true;
		}
		if(dzien <= 4.5){
			return false;
		}
		if(dzien > 4.5){
			throw new IllegalArgumentException("taka data nie istniała");
		}
		return false;
	}

	public static double dateToJulianDay1(YMD ymd) {
		int y = ymd.getYy();
		int m = ymd.getMm();
		double d = ymd.getDd();
		int a;
		int b = 0;
		if(m < 3){
			y--;
			m += 12;
		}
		if(isGCDate1(ymd)){
			a = y / 100;
			b = 2 - a + (a / 4);
		}
		return (int)(365.25 * (y + 4716)) + (int)(30.6001 * (m + 1)) + d
				+ b - 1524.5;
	}

	public static boolean isGCDate1(YMD ymd) {
		int rok = ymd.getYy();
		int miesiac = ymd.getMm();
		double dzien = ymd.getDd();
		if(rok > 1582){
			return true;
		}
		if(rok < 1582){
			return false;
		}
		if(miesiac > 10){
			return true;
		}
		if(miesiac < 10){
			return false;
		}
		if(dzien >= 15.5){
			return true;
		}
		if(dzien <= 4.5){
			return false;
		}
		if(dzien > 4.5){
			throw new IllegalArgumentException("taka data nie istniała");
		}
		return false;
	}

	public static boolean isValidYMDDate(int year, int month, double day) {
		if((month >= 1) && (month <= 12)){
			if(CU.dateToJulianDay(year, month, day) >= 0.0){
				if(!CU.isLeapYear(year)){
					return ((int) day >= 1) && ((int) day <= CU.LEN[month - 1]);
				}
				else{
					return ((int) day >= 1) && ((int) day <= CU.LEAP_LEN[month - 1]);
				}
			}
			else{
				throw new IllegalArgumentException(
						"data poza zakresem kalendarza");
			}
		}
		return false;
	}

	public static int amod(int index, int modulus) {
		int i = index;
		i = i % modulus;
		if(i == 0){
			i = modulus;
		}
		if(i < 0){
			i = (modulus + i);
		}
		return i;
	}

	//----------------------------------TZOLKIN
	public static boolean isValidTzolkin(int dd, String name) {
		return (isValidTzolkinDay(dd)) && (isValidTzolkinName(name));
	}

	public static int mmTzolkinNumber(String name)
			throws IllegalArgumentException {
		int monthNumber = -1;
		if(!isValidTzolkinName(name)){
			throw new IllegalArgumentException("niewlaściwa nazwa miesiąca");
		}
		for(int i = 0; i < TZOLKIN_MMS.length; i++){
			if(name.equals(TZOLKIN_MMS[i])){
				monthNumber = i + 1;
			}
		}
		return monthNumber;
	}

	public static boolean isValidTzolkinName(String name) {
		for(String s : TZOLKIN_MMS){
			if(name.equals(s)){
				return true;
			}
		}
		return false;
	}

	public static int tzolkinDayOfYear(int dd, String name)
			throws IllegalArgumentException {
		if(!isValidTzolkin(dd, name)){
			throw new IllegalArgumentException("niewłaściwy parametr");
		}
		int ind = -1;
		int mm = mmTzolkinNumber(name);
		for(int i = mm; i < TZOLKIN_YEAR_LENGTH + 1; i = i + TZOLKIN_MONTHS){
			ind = CU.amod(i, TZOLKIN_DAYS);
			if(dd == ind){
				return i;
			}
		}
		return ind;
	}

	/**
	 * Sprawdza czy numer dnia jest poprawny
	 * @param dd - numer dnia <1,13>
	 * @return zwraca true - jeśli nr dnia jest poprawny, false - jeśli nie jest
	 */
	public static boolean isValidTzolkinDay(int dd) {
		return (dd >= 1) && (dd <= TZOLKIN_DAYS);
	}

	/**
	 * Sprawdza czy dayOfYear jest poprawnym dniem w roku
	 * @param dayOfYear - nr kolejny dnia w orku
	 * @return zwraca true jeśli dayOfYear jest poprawny, false
	 * jeśli nie jest
	 */
	public static boolean isValidDayOfTzolkinYear(int dayOfYear) {
		return (dayOfYear >= 1) && (dayOfYear <= TZOLKIN_YEAR_LENGTH);
	}

	//------------------------ HAAB
	public static boolean isValidHaab(int dd, String name) {
		if(isValidHaabName(name)){
			int mm = mmHaabNumber(name);
			if(mm == HAAB_MAX_DAY_NO){
				return (dd >= 0) && (dd <= MAX_UAYEB_DAY_NO);
			}
			else{
				return isValidHaabDay(dd);
			}
		}
		return false;
	}

	private static boolean isValidHaabDay(int dd) {
		return (dd >= 0) && (dd <= HAAB_MAX_DAY_NO);
	}

	public static int mmHaabNumber(String name) throws IllegalArgumentException {
		int monthNumber = -1;
		if(!isValidHaabName(name)){
			throw new IllegalArgumentException("Niewłaściwa nazwa miesiąca");
		}
		for(int i = 0; i < HAAB_MMS.length; i++){
			if(name.equals(HAAB_MMS[i])){
				monthNumber = i + 1;
			}
		}
		return monthNumber;
	}

	public static int dayOfHaabYear(int dd, int mm) {
		if(!isValidHaab(dd, mm)){
			throw new IllegalArgumentException("argument poza zakresem");
		}
		return (mm - 1) * HAAB_DAYS + dd + 1;
	}

	public static boolean isValidHaab(int dd, int mm) {
		if(isValidHaabMonth(mm)){
			if(mm == HAAB_MAX_DAY_NO){
				return (dd >= 0) && (dd <= MAX_UAYEB_DAY_NO);
			}
			else{
				return isValidHaabDay(dd);
			}
		}
		return false;
	}

	private static boolean isValidHaabMonth(int mm) {
		return (mm >= 1) && (mm <= HAAB_MAX_DAY_NO);
	}

	public static boolean isValidHaabName(String name) {
		for(String s : HAAB_MMS){
			if(name.equals(s)){
				return true;
			}
		}
		return false;
	}

	public static boolean isValidDayOfHaabYear(int dayOfYear) {
		return (dayOfYear >= 1) && (dayOfYear <= HAAB_YEAR_LENGTH);
	}

	//------------------------------ LONG COUNT
	public static boolean isValidLCValue(int value) {
		return (value >= 0) && (value <= CU.MAX_LC_VALUE);
	}

	public static int[] lcValueToTable(int value) {
		if(!isValidLCValue(value)){
			throw new IllegalArgumentException("Zbyt wysoka liczba");
		}
		int[] lc = new int[5];
		lc[0] = (value / CU.LC_VALUES[0]);
		int baktun = value % CU.LC_VALUES[0];
		lc[1] = (baktun / CU.LC_VALUES[1]);
		int katun = baktun % CU.LC_VALUES[1];
		lc[2] = (katun / CU.LC_VALUES[2]);
		int tun = katun % CU.LC_VALUES[2];
		lc[3] = tun / CU.LC_VALUES[3];
		lc[4] = tun % CU.LC_VALUES[3];
		return lc;
	}

	public static String lcTtableToName(int[] table) {
		StringBuilder st = new StringBuilder();
		for(int i = 0; i < table.length; i++){
			if(i < table.length - 1){
				st.append(table[i]).append(".");
			}
			else{
				st.append(table[i]);
			}
		}
		return st.toString();
	}

	public static int nameToLCValue(String name) {
		int[] table = lcNameToTable(name);
		int value = tableToLCValue(table);
		if(value > CU.MAX_LC_VALUE){
			throw new IllegalArgumentException("nieprawidłowa wartość");
		}
		return value;
	}

	public static int[] lcNameToTable(String name) {
		StringTokenizer st = new StringTokenizer(name, ".");
		int[] table = new int[st.countTokens()];
		int i = 0;
		while(st.hasMoreTokens()){
			table[i] = Integer.parseInt(st.nextToken());
			i++;
		}
		return table;
	}

	public static int tableToLCValue(int[] table) {
		int result = 0;
		for(int i = 0; i < table.length; i++){
			result += table[i] * CU.LC_VALUES[i];
		}
		return result;
	}

	//------------------- Calendar Round
	public static boolean isValidCR(Tzolkin tzolkin, Haab haab) {
		int tzOffset = TZSTART.getDayOfYear() - 1;
		int haOffset = HASTART.getDayOfYear() - 1;
		int x;
		int y;
		for(int i = 1; i < CR_LENGTH + 1; i++){
			x = amod(i + tzOffset, CU.TZOLKIN_YEAR_LENGTH);
			y = amod(i + haOffset, CU.HAAB_YEAR_LENGTH);
			if((x == tzolkin.getDayOfYear()) && (y == haab.getDayOfYear())){
				return true;
			}
		}
		return false;
	}

	public static int dayOfCR(Tzolkin tzolkin, Haab haab) {
		int tzOffset = TZSTART.getDayOfYear() - 1;
		int haOffset = HASTART.getDayOfYear() - 1;
		int x;
		int y;
		for(int i = 1; i < CR_LENGTH + 1; i++){
			x = amod(i + tzOffset, CU.TZOLKIN_YEAR_LENGTH);
			y = amod(i + haOffset, CU.HAAB_YEAR_LENGTH);
			if((x == tzolkin.getDayOfYear()) && (y == haab.getDayOfYear())){
				return i;
			}
		}
		return -1;
	}

	public static MayaDate ymdToMayaDate(YMD ymd) {
		double jd = CU.dateToJulianDay1(ymd);
		double dif = jd - FIRST_MAYA_JD;
		LongCount lc = new LongCount((int)dif);
		CalendarRound cr = new CalendarRound();
		cr.addDays(lc.getValue());
		return new MayaDate(lc, cr);
	}
}
