package maya;

public class Tzolkin{
	private int dd;
	private int mm;
	private String name;
	private int dayOfYear;

	public Tzolkin(int dd, String name){
		if(!CU.isValidTzolkin(dd, name)){
			throw new IllegalArgumentException("Niewłaściwy argument");
		}
		this.dd = dd;
		this.name = name;
		this.mm = CU.mmTzolkinNumber(name);
		this.dayOfYear = CU.tzolkinDayOfYear(dd, name);
	}

	public Tzolkin(int dayOfYear){
		if(!CU.isValidDayOfTzolkinYear(dayOfYear)){
			throw new IllegalArgumentException("Niewłaściwa liczba dnia w roku");
		}
		this.dayOfYear = dayOfYear;
		this.dd = CU.amod(dayOfYear, CU.TZOLKIN_DAYS);
		this.mm = CU.amod(dayOfYear, CU.TZOLKIN_MONTHS);
		this.name = CU.TZOLKIN_MMS[mm - 1];
	}

	/**
	 * Pobiera aktualny dzień roku
	 * @return - zwraca numer kolejny dnia w roku
	 */
	public int getDayOfYear() {
		return dayOfYear;
	}

	/**
	 * Przesuwa ten kalendarz do przodu o wskazaną liczbę dni. Liczba dni nie
	 * może być ujemna. Można dodać dowolną liczbę dni.
	 * @param daysAdded - liczba dni, które należy dodać do tego kalendarza
	 */
	public void addDays(int daysAdded) {
		if(daysAdded < 0){
			throw new IllegalArgumentException(
					"liczba dni dodanych nie może być mniejsza od 0");
		}
		int tempDays = this.dayOfYear + daysAdded;
		int tempDayOfYear = CU.amod(tempDays, CU.TZOLKIN_YEAR_LENGTH);
		Tzolkin tempTz = new Tzolkin(tempDayOfYear);
		this.dayOfYear = tempTz.dayOfYear;
		this.dd = tempTz.dd;
		this.mm = tempTz.mm;
		this.name = tempTz.name;
	}

	/**
	 * Pobiera dzień miesiąca
	 * @return - zwraca aktualny dzień miesiąca
	 */
	public int getDd() {
		return dd;
	}

	/**
	 * Pobiera nazwę aktualnego miesiąca
	 * @return - zwraca nazwę aktualnego miesiąca
	 */
	public String getName() {
		return name;
	}

	@Override
	public String toString() {
		return dd + " " + name;
	}
}
